﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "BaseWeaponClass.h"

#include "AIHelpers.h"
#include "GeneratedCodeHelpers.h"
#include "EndlessVendetta/EndlessVendettaCharacter.h"
#include "Kismet/KismetMathLibrary.h"
#include "Components/CapsuleComponent.h"
#include "Kismet/GameplayStatics.h"


// Sets default values
ABaseWeaponClass::ABaseWeaponClass()
{
	// Set this actor to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;
}

// Called when the game starts or when spawned
void ABaseWeaponClass::BeginPlay()
{
	Super::BeginPlay();
	
	// Attempt to find the player character
	APlayerController* PlayerController = UGameplayStatics::GetPlayerController(this, 0);
	player = Cast<AEndlessVendettaCharacter>(PlayerController->GetCharacter());

	for (UActorComponent* actorComp : player->GetComponentsByTag(UArrowComponent::StaticClass(), FName("GunStart")))
	{
		GunStartArrow = Cast<UArrowComponent>(actorComp);
		break;
	}

	if (!playerControllerRef)
	{
		playerControllerRef = UGameplayStatics::GetPlayerController(GetWorld(), 0);
	}
	
}

// Called every frame
void ABaseWeaponClass::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);

	if (player)
	{
		ApplyRecoil(DeltaTime);
	}
}




// Recoil Handling //
void ABaseWeaponClass::GenerateRecoilVector()
{
	float angle = FMath::RandRange(recoilMaxAngleLeft, -recoilMaxAngleRight); //randomg recoil vector angle between left and right
	float recMag = recoilMagnitude * 252.f; //converting degrees to controller units
	float tempMag = -FMath::RandRange(recMag * recoilMinMultiplier, recMag); // recoil magnitude

	recoilResultYaw = FMath::Sin(FMath::DegreesToRadians(angle));
	recoilResultPitch = FMath::Cos(FMath::DegreesToRadians(angle));

	//scaling direction to magnitude
	recoilResultPitch *= -tempMag;
	recoilResultYaw *= tempMag;
	
	//reset recoil to 0 of curve
	recoilTime = 0;
}

void ABaseWeaponClass::ApplyRecoil(float DeltaTime)
{
	if (recoilTime < 0.3)
	{
		float amplitude = RecoilCurve->GetFloatValue(recoilTime); //get current value of curve in time
		recoilTime += DeltaTime;
		player->AddControllerPitchInput(GetRecoilPitch(amplitude, recoilTime));
		player->AddControllerYawInput(GetRecoilYaw(amplitude, recoilTime));
		UpdateSamples(amplitude, recoilTime);
	}
}

void ABaseWeaponClass::nullSamples()
{
	recoilCurvet = 0;
	recoilCurvez1 = 0;
}

float ABaseWeaponClass::GetRecoilPitch(float Amp, float Time)
{
	//Using the trapez method and we are getting the surface under the curve, each trapezoid consist of square and right triangle
	float lower = recoilCurvez1 < Amp ? recoilCurvez1 : Amp; //get which point is common for both triangle and square of trapezoid
	//lower point
	float mult = (Time - recoilCurvet) * lower; //getting surface of square
	mult += (Time - recoilCurvet) * (Amp - recoilCurvez1) / 2.0f; //getting and adding surface of triangle
	return (recoilResultPitch * mult) / playerControllerRef->InputPitchScale_DEPRECATED; //calculating and return recoil force for current frame
}


//Same as getrecoilpitch
float ABaseWeaponClass::GetRecoilYaw(float Amp, float Time)
{
	float lower = recoilCurvez1 < Amp ? recoilCurvez1 : Amp;
	float mult = (Time - recoilCurvet) * lower;

	mult += (Time - recoilCurvet) * (Amp - recoilCurvez1) / 2.0f;
	return (recoilResultYaw * mult) / playerControllerRef->InputYawScale_DEPRECATED;
}

void ABaseWeaponClass::UpdateSamples(float Amp, float Time)
{
	recoilCurvez1 = Amp;
	recoilCurvet = Time;
}


// Fire handling //
void ABaseWeaponClass::ClickDetectionTimer()
{
	GetWorldTimerManager().SetTimer(timerHandle, this, &ABaseWeaponClass::Fire, 1 / FireRate, false);	
}

void ABaseWeaponClass::CancelFire()
{
	GetWorldTimerManager().ClearTimer(timerHandle);
}

void ABaseWeaponClass::Fire()
{
	//do damage fallof based off distance
	FHitResult outHit;
	FVector traceStart;
	FVector traceEnd;
	traceStart = GunStartArrow->GetComponentLocation();
	traceEnd = traceStart + (GunStartArrow->GetForwardVector() * BulletDistance);
	FCollisionQueryParams collisionParams;
	collisionParams.AddIgnoredActor(player);
	collisionParams.AddIgnoredActor(this);
	if (GetWorldTimerManager().IsTimerActive(timerHandle))
	{
		return;
	}
	GetWorld()->LineTraceSingleByChannel(outHit, traceStart, traceEnd, ECC_Visibility, collisionParams);
	//Debug line to see where the trace hit
	DrawDebugLine(this->GetWorld(), traceStart, traceEnd, FColor::Red, false, 5.0f, 0U, 2.5f);
	playerControllerRef->PlayerCameraManager->StartCameraShake(CameraShakeClass, 1);
	GenerateRecoilVector();
	ClickDetectionTimer();
	if (outHit.bBlockingHit)
	{
		UE_LOG(LogTemp, Display, TEXT("hit item: %s"), *outHit.GetActor()->GetName());
	}
}
 
